@@ -20,20 +20,21 @@ def test_update_command(project, invoke, mocker):
 @pytest.mark.usefixtures("working_set")
 def test_update_ignore_constraints(project, repository):
     actions.do_add(project, packages=("pytz",))
-    assert project.meta.dependencies == ["pytz~=2019.3"]
+    assert project.pyproject.metadata["dependencies"] == ["pytz~=2019.3"]
     repository.add_candidate("pytz", "2020.2")
 
     actions.do_update(project, unconstrained=False, packages=("pytz",))
-    assert project.meta.dependencies == ["pytz~=2019.3"]
+    assert project.pyproject.metadata["dependencies"] == ["pytz~=2019.3"]
     assert project.locked_repository.all_candidates["pytz"].version == "2019.3"
 
     actions.do_update(project, unconstrained=True, packages=("pytz",))
-    assert project.meta.dependencies == ["pytz~=2020.2"]
+    assert project.pyproject.metadata["dependencies"] == ["pytz~=2020.2"]
     assert project.locked_repository.all_candidates["pytz"].version == "2020.2"
 
 
 @pytest.mark.usefixtures("working_set")
-def test_update_all_packages(project, repository, capsys):
+@pytest.mark.parametrize("strategy", ["reuse", "all"])
+def test_update_all_packages(project, repository, capsys, strategy):
     actions.do_add(project, packages=["requests", "pytz"])
     repository.add_candidate("pytz", "2019.6")
     repository.add_candidate("chardet", "3.0.5")
@@ -48,13 +49,16 @@ def test_update_all_packages(project, repository, capsys):
             "urllib3<1.24,>=1.21.1",
         ],
     )
-    actions.do_update(project)
+    actions.do_update(project, strategy=strategy)
     locked_candidates = project.locked_repository.all_candidates
     assert locked_candidates["requests"].version == "2.20.0"
-    assert locked_candidates["chardet"].version == "3.0.5"
+    assert locked_candidates["chardet"].version == (
+        "3.0.5" if strategy == "all" else "3.0.4"
+    )
     assert locked_candidates["pytz"].version == "2019.6"
     out, _ = capsys.readouterr()
-    assert "3 to update" in out, out
+    update_num = 3 if strategy == "all" else 2
+    assert f"{update_num} to update" in out, out
 
     actions.do_sync(project)
     out, _ = capsys.readouterr()
@@ -78,12 +82,12 @@ def test_update_dry_run(project, repository, capsys):
         ],
     )
     actions.do_update(project, dry_run=True)
-    project.lockfile = None
+    out, _ = capsys.readouterr()
+    project.lockfile.reload()
     locked_candidates = project.locked_repository.all_candidates
     assert locked_candidates["requests"].version == "2.19.1"
     assert locked_candidates["chardet"].version == "3.0.4"
     assert locked_candidates["pytz"].version == "2019.3"
-    out, _ = capsys.readouterr()
     assert "requests 2.19.1 -> 2.20.0" in out
 
 
@@ -176,15 +180,29 @@ def test_update_with_prerelease_without_package_argument(project):
 @pytest.mark.usefixtures("repository")
 def test_update_existing_package_with_prerelease(project, working_set):
     actions.do_add(project, packages=["urllib3"])
-    assert project.meta.dependencies[0] == "urllib3~=1.22"
+    assert project.pyproject.metadata["dependencies"][0] == "urllib3~=1.22"
     assert working_set["urllib3"].version == "1.22"
 
     actions.do_update(project, packages=["urllib3"], prerelease=True)
-    assert project.meta.dependencies[0] == "urllib3~=1.22"
+    assert project.pyproject.metadata["dependencies"][0] == "urllib3~=1.22"
     assert working_set["urllib3"].version == "1.23b0"
 
     actions.do_update(
         project, packages=["urllib3"], prerelease=True, unconstrained=True
     )
-    assert project.meta.dependencies[0] == "urllib3<2,>=1.23b0"
+    assert project.pyproject.metadata["dependencies"][0] == "urllib3<2,>=1.23b0"
     assert working_set["urllib3"].version == "1.23b0"
+
+
+def test_update_package_with_extras(project, repository, working_set):
+    repository.add_candidate("foo", "0.1")
+    foo_deps = ["urllib3; extra == 'req'"]
+    repository.add_dependencies("foo", "0.1", foo_deps)
+    actions.do_add(project, packages=["foo[req]"])
+    assert working_set["foo"].version == "0.1"
+
+    repository.add_candidate("foo", "0.2")
+    repository.add_dependencies("foo", "0.2", foo_deps)
+    actions.do_update(project)
+    assert working_set["foo"].version == "0.2"
+    assert project.locked_repository.all_candidates["foo"].version == "0.2"
