@@ -6,7 +6,7 @@ from pathlib import Path
 import pytest
 
 from pdm.cli import actions
-from pdm.exceptions import InvalidPyVersion
+from pdm.exceptions import NoPythonVersion
 from pdm.models.caches import JSONFileCache
 
 
@@ -20,9 +20,7 @@ def test_use_command(project, invoke):
 
     result = invoke(["use", "-f", python_path], obj=project)
     assert result.exit_code == 0
-
-    project.meta["requires-python"] = ">=3.6"
-    project.write_pyproject()
+    project.pyproject.metadata["requires-python"] = ">=3.6"
     result = invoke(["use", "2.7"], obj=project)
     assert result.exit_code == 1
 
@@ -56,7 +54,7 @@ echo hello
     shim_path = project.root.joinpath("python_shim.sh")
     shim_path.write_text(wrapper_script)
     shim_path.chmod(0o755)
-    with pytest.raises(InvalidPyVersion):
+    with pytest.raises(NoPythonVersion):
         actions.do_use(project, shim_path.as_posix())
 
 
@@ -68,7 +66,7 @@ def test_use_remember_last_selection(project, mocker):
     assert not cache._cache
     actions.do_use(project, "3", first=True)
     cache._read_cache()
-    assert cache.has_key("3")
+    assert "3" in cache
     mocker.patch.object(project, "find_interpreters")
     actions.do_use(project, "3")
     project.find_interpreters.assert_not_called()
