@@ -56,7 +56,7 @@ def test_project_sources_overriding(project):
     project.project_config["pypi.url"] = "https://test.pypi.org/simple"
     assert project.sources[0]["url"] == "https://test.pypi.org/simple"
 
-    project.tool_settings["source"] = [
+    project.pyproject.settings["source"] = [
         {"url": "https://example.org/simple", "name": "pypi", "verify_ssl": True}
     ]
     assert project.sources[0]["url"] == "https://example.org/simple"
@@ -76,7 +76,7 @@ def test_project_sources_env_var_expansion(project, monkeypatch):
         == "https://${PYPI_USER}:${PYPI_PASS}@test.pypi.org/simple"
     )
 
-    project.tool_settings["source"] = [
+    project.pyproject.settings["source"] = [
         {
             "url": "https://${PYPI_USER}:${PYPI_PASS}@example.org/simple",
             "name": "pypi",
@@ -87,11 +87,11 @@ def test_project_sources_env_var_expansion(project, monkeypatch):
     assert project.sources[0]["url"] == "https://user:password@example.org/simple"
     # not expanded in tool settings
     assert (
-        project.tool_settings["source"][0]["url"]
+        project.pyproject.settings["source"][0]["url"]
         == "https://${PYPI_USER}:${PYPI_PASS}@example.org/simple"
     )
 
-    project.tool_settings["source"] = [
+    project.pyproject.settings["source"] = [
         {
             "url": "https://${PYPI_USER}:${PYPI_PASS}@example2.org/simple",
             "name": "example2",
@@ -102,7 +102,7 @@ def test_project_sources_env_var_expansion(project, monkeypatch):
     assert project.sources[1]["url"] == "https://user:password@example2.org/simple"
     # not expanded in tool settings
     assert (
-        project.tool_settings["source"][0]["url"]
+        project.pyproject.settings["source"][0]["url"]
         == "https://${PYPI_USER}:${PYPI_PASS}@example2.org/simple"
     )
 
@@ -178,12 +178,15 @@ def test_ignore_saved_python(project, monkeypatch):
 
 
 def test_select_dependencies(project):
-    project.meta["dependencies"] = ["requests"]
-    project.meta["optional-dependencies"] = {
+    project.pyproject.metadata["dependencies"] = ["requests"]
+    project.pyproject.metadata["optional-dependencies"] = {
         "security": ["cryptography"],
         "venv": ["virtualenv"],
     }
-    project.tool_settings["dev-dependencies"] = {"test": ["pytest"], "doc": ["mkdocs"]}
+    project.pyproject.settings["dev-dependencies"] = {
+        "test": ["pytest"],
+        "doc": ["mkdocs"],
+    }
     assert sorted(project.get_dependencies()) == ["requests"]
     assert sorted(project.dependencies) == ["requests"]
 
@@ -231,13 +234,16 @@ def test_create_venv_first_time(invoke, project, local_finder):
     assert Path(project.project_config["python.path"]).relative_to(venv_path)
 
 
-@pytest.mark.usefixtures("venv_backends")
-def test_create_venv_in_project(invoke, project, local_finder):
-    project.project_config.update({"venv.in_project": True})
+@pytest.mark.usefixtures("venv_backends", "local_finder")
+@pytest.mark.parametrize("with_pip", [True, False])
+def test_create_venv_in_project(invoke, project, with_pip):
+    project.project_config.update({"venv.in_project": True, "venv.with_pip": with_pip})
     del project.project_config["python.path"]
     result = invoke(["install"], obj=project)
     assert result.exit_code == 0
     assert project.root.joinpath(".venv").exists()
+    working_set = project.environment.get_working_set()
+    assert ("pip" in working_set) is with_pip
 
 
 @pytest.mark.usefixtures("venv_backends")
